import logging

from spaceone.core.manager import BaseManager
from spaceone.core.connector.space_connector import SpaceConnector

_LOGGER = logging.getLogger(__name__)


class IdentityManager(BaseManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.identity_connector: SpaceConnector = self.locator.get_connector(
            SpaceConnector, service="identity"
        )

    def check_workspace(self, workspace_id, domain_id):
        return self.identity_connector.dispatch(
            "Workspace.check", {"workspace_id": workspace_id, "domain_id": domain_id}
        )



from google.protobuf.empty_pb2 import Empty
from spaceone.core.pygrpc.message_type import *

__all__ = ["EmptyInfo", "StatisticsInfo"]


def EmptyInfo():
    return Empty()


def StatisticsInfo(result):
    return change_struct_type(result)



import logging
from typing import Tuple, Union

from spaceone.core.service import *
from spaceone.file_manager.error import *
from spaceone.file_manager.model.file_model import File
from spaceone.file_manager.manager.file_manager import FileManager
from spaceone.file_manager.manager.file_connector_manager import FileConnectorManager
from spaceone.file_manager.manager.identity_manager import IdentityManager

_LOGGER = logging.getLogger(__name__)


@authentication_handler
@authorization_handler
@mutation_handler
@event_handler
class FileService(BaseService):
    resource = "File"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.file_mgr: FileManager = self.locator.get_manager(FileManager)
        self.identity_mgr: IdentityManager = self.locator.get_manager(IdentityManager)

    @transaction(
        permission="file-manager:File.write",
        role_types=["SYSTEM_ADMIN", "DOMAIN_ADMIN", "WORKSPACE_OWNER"],
    )
    @check_required(["name", "resource_group"])
    def add(self, params: dict) -> Tuple[File, str, dict]:
        """Add file

        Args:
            params (dict): {
                'name': 'str',              # required
                'tags': 'dict',
                'reference': 'dict',
                'resource_group',           # required
                'workspace_id': 'str',
                'domain_id': 'str'
            }

        Returns:
            file_vo
        """
        resource_group = params["resource_group"]
        workspace_id = params.get("workspace_id")
        domain_id = params.get("domain_id")
        params["file_type"] = self._get_file_type(params["name"])

        if resource_group == "SYSTEM":
            params["domain_id"] = "*"
            params["workspace_id"] = "*"
        elif resource_group == "DOMAIN":
            params["workspace_id"] = "*"
        else:
            self.identity_mgr.check_workspace(workspace_id, domain_id)

        file_vo: File = self.file_mgr.create_file(params)

        file_conn_mgr: FileConnectorManager = self.locator.get_manager(
            FileConnectorManager
        )
        upload_url, upload_options = file_conn_mgr.get_upload_url(
            file_vo.file_id, file_vo.name
        )

        return file_vo, upload_url, upload_options

    @transaction(
        permission="file-manager:File.write",
        role_types=["SYSTEM_ADMIN", "DOMAIN_ADMIN", "WORKSPACE_OWNER"],
    )
    @check_required(["file_id"])
    def update(self, params: dict) -> File:
        """Update file

        Args:
            params (dict): {
                'file_id': 'str',        # required
                'tags': 'dict',
                'reference': 'dict',
                'workspace_id': 'str',   # injected from auth
                'domain_id': 'str'       # injected from auth
            }

        Returns:
            file_vo
        """

        workspace_id = params.get("workspace_id")
        domain_id = params.get("domain_id")

        file_id = params["file_id"]
        file_vo: File = self.file_mgr.get_file(file_id, workspace_id, domain_id)
        file_vo = self.file_mgr.update_file_by_vo(params, file_vo)

        return file_vo

    @transaction(
        permission="file-manager:File.write",
        role_types=["SYSTEM_ADMIN", "DOMAIN_ADMIN", "WORKSPACE_OWNER"],
    )
    @check_required(["file_id"])
    def delete(self, params: dict) -> None:
        """Delete file

        Args:
            params (dict): {
                'file_id': 'str',        # required
                'workspace_id': 'str',   # injected from auth
                'domain_id': 'str'       # injected from auth
            }

        Returns:
            None
        """

        file_id = params["file_id"]
        workspace_id = params.get("workspace_id")
        domain_id = params.get("domain_id")

        file_vo: File = self.file_mgr.get_file(file_id, workspace_id, domain_id)

        file_conn_mgr: FileConnectorManager = self.locator.get_manager(
            FileConnectorManager
        )
        file_conn_mgr.delete_file(file_id, file_vo.name)

        self.file_mgr.delete_file_by_vo(file_vo)

    @transaction(
        permission="file-manager:File.read",
        role_types=[
            "SYSTEM_ADMIN",
            "DOMAIN_ADMIN",
            "WORKSPACE_OWNER",
            "WORKSPACE_MEMBER",
        ],
    )
    @change_value_by_rule("APPEND", "domain_id", "*")
    @change_value_by_rule("APPEND", "workspace_id", "*")
    @change_value_by_rule("APPEND", "user_projects", "*")
    @check_required(["file_id"])
    def get_download_url(self, params: dict) -> Tuple[File, Union[str, None]]:
        """Get download url of file

        Args:
            params (dict): {
                'file_id': 'str',         # required
                'workspace_id': 'str',    # injected from auth
                'domain_id': 'str'        # injected from auth
                'user_projects': 'list'   # injected from auth
            }

        Returns:
            file_data (dict)
        """

        file_id = params["file_id"]
        workspace_id = params.get("workspace_id")
        domain_id = params.get("domain_id")

        file_vo: File = self.file_mgr.get_file(file_id, workspace_id, domain_id)

        file_conn_mgr: FileConnectorManager = self.locator.get_manager(
            FileConnectorManager
        )

        if file_vo.state == "PENDING":
            if not file_conn_mgr.check_file(file_id, file_vo.name):
                raise ERROR_FILE_UPLOAD_STATE()

            conditions = {"state": "DONE"}
            if workspace_id:
                conditions["workspace_id"] = workspace_id
            if domain_id:
                conditions["domain_id"] = domain_id

            file_vo = self.file_mgr.update_file_by_vo(conditions, file_vo)

        download_url = file_conn_mgr.get_download_url(
            file_id, file_vo.name, file_vo.domain_id
        )

        return file_vo, download_url

    @transaction(
        permission="file-manager:File.read",
        role_types=[
            "SYSTEM_ADMIN",
            "DOMAIN_ADMIN",
            "WORKSPACE_OWNER",
            "WORKSPACE_MEMBER",
        ],
    )
    @change_value_by_rule("APPEND", "domain_id", "*")
    @change_value_by_rule("APPEND", "workspace_id", "*")
    @change_value_by_rule("APPEND", "user_projects", "*")
    @check_required(["file_id"])
    def get(self, params: dict) -> File:
        """Get file

        Args:
            params (dict): {
                'file_id': 'str',          # required
                'workspace_id': 'str',     # injected from auth
                'domain_id': 'str'         # injected from auth
                'user_projects': 'list'    # injected from auth
            }

        Returns:
            file_vo
        """

        file_id = params["file_id"]
        workspace_id = params.get("workspace_id")
        domain_id = params.get("domain_id")
        user_projects = params.get("user_projects")

        return self.file_mgr.get_file(file_id, workspace_id, domain_id, user_projects)

    @transaction(
        permission="file-manager:File.read",
        role_types=[
            "SYSTEM_ADMIN",
            "DOMAIN_ADMIN",
            "WORKSPACE_OWNER",
            "WORKSPACE_MEMBER",
        ],
    )
    @change_value_by_rule("APPEND", "domain_id", "*")
    @change_value_by_rule("APPEND", "workspace_id", "*")
    @change_value_by_rule("APPEND", "user_projects", "*")
    @append_query_filter(
        [
            "file_id",
            "name",
            "state",
            "file_type",
            "resource_type",
            "resource_id",
            "resource_group",
            "workspace_id",
            "domain_id",
            "user_projects",
        ]
    )
    @append_keyword_filter(["file_id", "name"])
    def list(self, params: dict) -> dict:
        """List files

        Args:
            params (dict): {
                'query': 'dict (spaceone.api.core.v1.Query)',
                'file_id': 'str',
                'name': 'str',
                'state': 'str',
                'file_type': 'str',
                'resource_type': 'str',
                'resource_id': 'str',
                'resource_group': 'str',
                'workspace_id': 'str',
                'domain_id': 'str'
                'user_projects': 'list'                             # injected from auth
            }

        Returns:
            results (list)
            total_count (int)
        """

        query = params.get("query", {})
        return self.file_mgr.list_files(query)

    @transaction(
        permission="file-manager:File.read",
        role_types=[
            "SYSTEM_ADMIN",
            "DOMAIN_ADMIN",
            "WORKSPACE_OWNER",
            "WORKSPACE_MEMBER",
        ],
    )
    @change_value_by_rule("APPEND", "domain_id", "*")
    @change_value_by_rule("APPEND", "workspace_id", "*")
    @change_value_by_rule("APPEND", "user_projects", "*")
    @check_required(["query"])
    @append_query_filter(["user_projects", "workspace_id", "domain_id"])
    @append_keyword_filter(["file_id", "name"])
    def stat(self, params: dict) -> dict:
        """
        Args:
            params (dict): {
                'query': 'dict (spaceone.api.core.v1.StatisticsQuery)',  # required
                'workspace_id': 'str',
                'domain_id': 'str'(meta),                                # required
            }

        Returns:
            values (list) : 'list of statistics data'

        """

        query = params.get("query", {})
        return self.file_mgr.stat_files(query)

    @staticmethod
    def _get_file_type(file_name: str) -> Union[str, None]:
        file_name_split = file_name.split(".")
        if len(file_name_split) == 1:
            return None
        else:
            return file_name_split[-1]



import abc
from spaceone.core.connector import BaseConnector


class FileBaseConnector(BaseConnector):
    @abc.abstractmethod
    def get_upload_url(self, file_id: str, file_name: str) -> [str, dict]:
        pass

    @abc.abstractmethod
    def get_download_url(self, file_id: str, file_name: str) -> str:
        pass

    @abc.abstractmethod
    def check_file(self, file_id: str, file_name: str) -> bool:
        pass

    @abc.abstractmethod
    def delete_file(self, file_id: str, file_name: str) -> None:
        pass



from spaceone.api.file_manager.v1 import file_pb2, file_pb2_grpc
from spaceone.core.pygrpc import BaseAPI
from spaceone.file_manager.service.file_service import FileService
from spaceone.file_manager.info.file_info import *
from spaceone.file_manager.info.common_info import *


class File(BaseAPI, file_pb2_grpc.FileServicer):
    pb2 = file_pb2
    pb2_grpc = file_pb2_grpc

    def add(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service(FileService, metadata) as file_service:
            file_vo, upload_url, upload_options = file_service.add(params)
            return self.locator.get_info(
                FileInfo, file_vo, upload_url=upload_url, upload_options=upload_options
            )

    def update(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service(FileService, metadata) as file_service:
            return self.locator.get_info(FileInfo, file_service.update(params))

    def delete(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service(FileService, metadata) as file_service:
            file_service.delete(params)
            return self.locator.get_info(EmptyInfo)

    def get_download_url(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service(FileService, metadata) as file_service:
            file_vo, download_url = file_service.get_download_url(params)
            return self.locator.get_info(FileInfo, file_vo, download_url=download_url)

    def get(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service(FileService, metadata) as file_service:
            return self.locator.get_info(FileInfo, file_service.get(params))

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service(FileService, metadata) as file_service:
            file_vos, total_count = file_service.list(params)
            return self.locator.get_info(
                FilesInfo, file_vos, total_count, minimal=self.get_minimal(params)
            )

    def stat(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service(FileService, metadata) as file_service:
            return self.locator.get_info(StatisticsInfo, file_service.stat(params))



import base64
import os
import json
import collections

from cryptography.hazmat.primitives.ciphers.aead import AESGCM
from spaceone.core import config
from spaceone.core.handler import BaseMutationHandler
from spaceone.secret.error.custom import ERROR_WRONG_ENCRYPT_ALGORITHM

__all__ = ["EncryptMutationHandler"]

_encrypt_verb = ["create", "update_data"]
_decrypt_verb = ["get_data"]

_SUPPORTED_ENCRYPT_ALGORITHM = ["AES"]


class EncryptMutationHandler(BaseMutationHandler):
    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)
        self.encrypt_algorithm = self.config.get("ENCRYPT_ALGORITHM")
        self._check_config(encrypt_algorithm)

    def request(self, params: dict) -> dict:
        if not (
            self.transaction.resource == "Secret"
            and self.transaction.verb in _encrypt_verb
        ):
            return params

        encrypted_data, nonce = self._encrypt(params["data"])
        params["data"] = {"encrypted_data": encrypted_data}
        params["encrypted"] = (True,)
        params["encrypt_options"] = {
            "encrypt_algorithm": self.encrypt_algorithm,
            "nonce": nonce,
        }
        return params

    def response(self, result: dict) -> dict:
        if not (
            self.transaction.resource == "Secret"
            and self.transaction.verb in _decrypt_verb
        ):
            return result

        if result["encrypted"] is not True:
            return result

        encrypt_options = result.get("encrypt_options", {})
        encrypt_algorithm = encrypt_options.get("encrypt_algorithm")
        if encrypt_algorithm not in _SUPPORTED_ENCRYPT_ALGORITHM:
            raise ERROR_WRONG_ENCRYPT_ALGORITHM(
                encrypt_algorithm=str(encrypt_algorithm)
            )

        plain_data = self._decrypt(result["data"]["encrypted_data"], encrypt_options)

        result = {"encrypted": False, "encrypt_options": {}, "data": plain_data}

        return result

    def _check_config(self, encrypt_algorithm: str) -> None:
        if encrypt_algorithm not in _SUPPORTED_ENCRYPT_ALGORITHM:
            raise ERROR_WRONG_ENCRYPT_ALGORITHM(encrypt_algorithm)

    def _dict_to_b64(self, data: dict) -> bytes:
        return base64.b64encode(json.dumps(data).encode())

    def _b64_to_dict(self, data):
        _data = data if isinstance(data, bytes) else data.encode()
        return json.loads(base64.b64decode(_data).decode())

    def _encrypt(self, plain_data: dict) -> (str, str):
        nonce = os.urandom(12)
        data_key = base64.b64decode(self.config.get("encrypt_key"))

        aesgcm = AESGCM(data_key)
        del data_key
        encrypted_data = aesgcm.encrypt(nonce, self._dict_to_b64(plain_data), None)

        encrypted_data_b64 = base64.b64encode(encrypted_data).decode()
        nonce_b64 = base64.b64encode(nonce).decode()
        return encrypted_data_b64, nonce_b64

    def _decrypt(self, encrypted_data: str, encrypt_options: dict) -> dict:
        nonce = base64.b64decode(encrypt_options["nonce"])
        data_key = base64.b64decode(self.config.get("encrypt_key"))

        aesgcm = AESGCM(data_key)
        del data_key

        plain_data = self._b64_to_dict(
            aesgcm.decrypt(nonce, base64.b64decode(encrypted_data), None)
        )

        return plain_data
